#!/usr/bin/env perl

#
#
# This Script is used to remove LaTeX comments from the .cxx source
# files in the Insight/Examples directory.
# 
# This is helpful to recover only plain C++ code and remove all 
# content intended for the complementary script ParserCxxExamples.pl.
#
#


if( $#ARGV < 1 )
  {
  print "Usage: ".$ARG0." <input file> <output file> \n";
  exit;
  }



ParseCxxFile( $ARGV[0] , $ARGV[1] );







#
#  Subroutine for parsing each one of the .cxx files.
#  A .cxx file is produced as a result
#
sub ParseCxxFile {

  my $inputfilename  = shift;
  my $outputfilename = shift;

  # truncate the initial part of the path
  $inputfilename =~ /(Examples\/.*$)/;
  my $examplefilename = $1;

  print "Processing $inputfilename into $outputfilename  ... \n";

  open(INFILE,    "$inputfilename"  )  or die "Can't open $inputfilename $!";
  open(OUTFILE,  ">$outputfilename" )  or die "Can't open $outputfilename $!";

  my $beginlatextag = "BeginLatex";
  my $endlatextag   = "EndLatex";

  my $begincodesnippettag = "BeginCodeSnippet";
  my $endcodesnippettag   = "EndCodeSnippet";

  my $dumpingcode  = 1;



  # The following message is a warning written on the generated .tex
  # files for preventing them from being manualy edited.
  print OUTFILE "\/\/ Please do NOT edit this file.\n";
  print OUTFILE "\/\/ It has been automatically generated\n";
  print OUTFILE "\/\/ by a perl script from the original cxx sources\n";
  print OUTFILE "\/\/ in the Insight/Examples directory\n\n";
  print OUTFILE "\/\/ Any changes should be made in the file\n";
  print OUTFILE "\/\/ $inputfilename\n";
  print OUTFILE "\n\n";



  while(<INFILE>) {

    my $tagfound     = 0;

    if( /$beginlatextag/ ) {
      $tagfound = 1;
      $dumpingcode  = 0;
      }
    elsif( /$begincodesnippettag/ ) {
      $tagfound = 1;
      $dumpingcode  = 1;
      }
    elsif( /$endlatextag/ ) {
      $tagfound = 1;
      $dumpingcode  = 1;
      }
    elsif( /$endcodesnippettag/ ) {
      $tagfound = 1;
      $dumpingcode = 1;
      }
    if( !$tagfound ) {
      if( $dumpingcode ) {
        print OUTFILE "$_";
        }
      }

  }

}


